// ignore_for_file: non_constant_identifier_names, camel_case_types, missing_return, unused_import, unused_local_variable, dead_code, unnecessary_cast
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:tencent_live_fluttify/src/android/android.export.g.dart';
import 'package:tencent_live_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';

part 'pusher_delegates.dart';

/// 推流控制器
class LivePusher extends _Holder with _Community, _Pro {
  LivePusher._();

  static Future<LivePusher> create({bool enableTouchFocus}) async {
    return platform(
      android: (pool) async {
        final instance = LivePusher._();

        final config = await com_tencent_rtmp_TXLivePushConfig.create__();
        if (enableTouchFocus != null) {
          await config.setTouchFocus(enableTouchFocus);
        }

        final context = await android_app_Application.get();
        instance._androidPusher = await com_tencent_rtmp_TXLivePusher
            .create__android_content_Context(context);

        await instance._androidPusher.setConfig(config);

        return instance;
      },
      ios: (pool) async {
        final instance = LivePusher._();

        final config = await TXLivePushConfig.create__();
        if (enableTouchFocus != null) {
          await config.set_touchFocus(enableTouchFocus);
        }

        instance._iosPusher = await TXLivePush.create__();

        await instance._iosPusher.initWithConfig(config);
        return instance;
      },
    );
  }

  /// 事件处理
  ///
  /// [onWaringNetBusy]弱网提醒
  Future<void> setOnEventListener({
    VoidCallback onWaringNetBusy,
    VoidCallback onWaringReconnect,
    VoidCallback onWaringHardwareAccelerationFail,
    VoidCallback onWaringDNSFail,
    VoidCallback onWaringServerConnFail,
    VoidCallback onWaringShakeFail,
    VoidCallback onWaringServerDisconnect,
    VoidCallback onEventConnectSucc,
    VoidCallback onEventPushBegin,
    VoidCallback onEventOpenCameraSuccess,
    VoidCallback onErrorOpenCameraFail,
    VoidCallback onErrorOpenMicFail,
    VoidCallback onErrorVideoEncodeFail,
    VoidCallback onErrorAudioEncodeFail,
    VoidCallback onErrorUnsupportedResolution,
    VoidCallback onErrorUnsupportedSampleRate,
    VoidCallback onErrorNetDisconnect,
  }) async {
    return platform(
      android: (pool) async {
        _androidPusher.setPushListener(
          _androidEventDelegate
            .._onWaringNetBusy = onWaringNetBusy
            .._onWaringReconnect = onWaringReconnect
            .._onWaringHardwareAccelerationFail =
                onWaringHardwareAccelerationFail
            .._onWaringDNSFail = onWaringDNSFail
            .._onWaringServerConnFail = onWaringServerConnFail
            .._onWaringShakeFail = onWaringShakeFail
            .._onWaringServerDisconnect = onWaringServerDisconnect
            .._onEventConnectSucc = onEventConnectSucc
            .._onEventPushBegin = onEventPushBegin
            .._onEventOpenCameraSuccess = onEventOpenCameraSuccess
            .._onErrorOpenCameraFail = onErrorOpenCameraFail
            .._onErrorOpenMicFail = onErrorOpenMicFail
            .._onErrorVideoEncodeFail = onErrorVideoEncodeFail
            .._onErrorAudioEncodeFail = onErrorAudioEncodeFail
            .._onErrorUnsupportedResolution = onErrorUnsupportedResolution
            .._onErrorUnsupportedSampleRate = onErrorUnsupportedSampleRate
            .._onErrorNetDisconnect = onErrorNetDisconnect,
        );
      },
      ios: (pool) async {
        _iosPusher.set_delegate(
          _iosEventDelegate
            .._onWaringNetBusy = onWaringNetBusy
            .._onWaringReconnect = onWaringReconnect
            .._onWaringHardwareAccelerationFail =
                onWaringHardwareAccelerationFail
            .._onWaringDNSFail = onWaringDNSFail
            .._onWaringServerConnFail = onWaringServerConnFail
            .._onWaringShakeFail = onWaringShakeFail
            .._onWaringServerDisconnect = onWaringServerDisconnect
            .._onEventConnectSucc = onEventConnectSucc
            .._onEventPushBegin = onEventPushBegin
            .._onEventOpenCameraSuccess = onEventOpenCameraSuccess
            .._onErrorOpenCameraFail = onErrorOpenCameraFail
            .._onErrorOpenMicFail = onErrorOpenMicFail
            .._onErrorVideoEncodeFail = onErrorVideoEncodeFail
            .._onErrorAudioEncodeFail = onErrorAudioEncodeFail
            .._onErrorUnsupportedResolution = onErrorUnsupportedResolution
            .._onErrorUnsupportedSampleRate = onErrorUnsupportedSampleRate
            .._onErrorNetDisconnect = onErrorNetDisconnect,
        );
      },
    );
  }
}

mixin _Community on _Holder {
  /// 开始预览
  Future<void> startPreview(dynamic preview) async {
    assert(
      preview is com_tencent_rtmp_ui_TXCloudVideoView || preview is UIView,
    );
    return platform(
      android: (pool) async {
        await _androidPusher.startCameraPreview(preview);
      },
      ios: (pool) async {
        await _iosPusher.startPreview(preview);
      },
    );
  }

  /// 开始推流
  Future<void> startPush(String pushUrl) async {
    return platform(
      android: (pool) async {
        final result = await _androidPusher.startPusher(pushUrl);
        debugPrint('result: $result');
      },
      ios: (pool) async {
        final result = await _iosPusher.startPush(pushUrl);
        debugPrint('result: $result');
      },
    );
  }

  /// 暂停推流
  Future<void> pausePush() async {
    return platform(
      android: (pool) => _androidPusher.pausePusher(),
      ios: (pool) => _iosPusher.pausePush(),
    );
  }

  /// 停止预览
  Future<void> stopPreview() async {
    return platform(
      android: (pool) => _androidPusher.stopCameraPreview(true),
      ios: (pool) => _iosPusher.stopPreview(),
    );
  }

  /// 停止推流
  Future<void> stopPush() async {
    return platform(
      android: (pool) => _androidPusher.stopPusher(),
      ios: (pool) => _iosPusher.stopPush(),
    );
  }

  /// 切换前后摄像头
  Future<void> switchCamera() async {
    return platform(
      android: (pool) => _androidPusher.switchCamera(),
      ios: (pool) => _iosPusher.switchCamera(),
    );
  }

  /// 打开/关闭闪光灯
  ///
  /// 仅在当前是后置摄像头时有效
  Future<void> turnFlashLight(bool on) async {
    return platform(
      android: (pool) => _androidPusher.turnOnFlashLight(on),
      ios: (pool) => _iosPusher.toggleTorch(on),
    );
  }

  /// 播放背景音乐
  ///
  /// 调用 TXLivePush 中的 BGM 相关接口可以实现背景混音功能。背景混音是指主播在直播时可以选取一首歌曲伴唱，
  /// 歌曲会在主播的手机端播放出来，同时也会被混合到音视频流中被观众端听到，所以被称为“混音”
  Future<void> playBGM(String path) async {
    return platform(
      android: (pool) => _androidPusher.playBGM(path),
      ios: (pool) => _iosPusher.playBGM(path),
    );
  }

  /// 停止背景音乐
  Future<void> stopBGM() async {
    return platform(
      android: (pool) => _androidPusher.stopBGM(),
      ios: (pool) => _iosPusher.stopBGM(),
    );
  }

  /// 暂停背景音乐
  Future<void> pauseBGM() async {
    return platform(
      android: (pool) => _androidPusher.pauseBGM(),
      ios: (pool) => _iosPusher.pauseBGM(),
    );
  }

  /// 恢复背景音乐
  Future<void> resumeBGM() async {
    return platform(
      android: (pool) => _androidPusher.resumeBGM(),
      ios: (pool) => _iosPusher.resumeBGM(),
    );
  }

  /// 麦克风音量大小
  Future<void> setMicVolume(double volume) async {
    return platform(
      android: (pool) => _androidPusher.setMicVolume(volume),
      ios: (pool) => _iosPusher.setMicVolume(volume),
    );
  }

  /// 背景音乐音量大小
  Future<void> setBGMVolume(double volume) async {
    return platform(
      android: (pool) => _androidPusher.setBGMVolume(volume),
      ios: (pool) => _iosPusher.setBGMVolume(volume),
    );
  }

  /// 背景音乐音调高低
  Future<void> setBgmPitch(double level) async {
    return platform(
      android: (pool) => _androidPusher.setBGMPitch(level),
      ios: (pool) => _iosPusher.setBGMPitch(level),
    );
  }
}

mixin _Pro on _Holder {}

class _Holder {
  com_tencent_rtmp_TXLivePusher _androidPusher;
  TXLivePush _iosPusher;

  com_tencent_rtmp_TXLivePushConfig _androidConfig;
  TXLivePushConfig _iOSConfig;

  final _androidEventDelegate = _AndroidEventDelegate();
  final _iosEventDelegate = _IOSEventDelegate();
}
